@@ -14,7 +14,7 @@ from django.contrib.admin.utils import (
 from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import models
 from django.utils import timezone
-from django.utils.encoding import force_text, smart_text
+from django.utils.encoding import force_text
 from django.utils.translation import ugettext_lazy as _
 
 
@@ -207,7 +207,7 @@ class RelatedFieldListFilter(FieldListFilter):
         }
         for pk_val, val in self.lookup_choices:
             yield {
-                'selected': self.lookup_val == smart_text(pk_val),
+                'selected': self.lookup_val == force_text(pk_val),
                 'query_string': changelist.get_query_string({
                     self.lookup_kwarg: pk_val,
                 }, [self.lookup_kwarg_isnull]),
@@ -222,6 +222,7 @@ class RelatedFieldListFilter(FieldListFilter):
                 'display': self.empty_value_display,
             }
 
+
 FieldListFilter.register(lambda f: f.remote_field, RelatedFieldListFilter)
 
 
@@ -260,6 +261,7 @@ class BooleanFieldListFilter(FieldListFilter):
                 'display': _('Unknown'),
             }
 
+
 FieldListFilter.register(
     lambda f: isinstance(f, (models.BooleanField, models.NullBooleanField)),
     BooleanFieldListFilter
@@ -292,7 +294,7 @@ class ChoicesFieldListFilter(FieldListFilter):
                 none_title = title
                 continue
             yield {
-                'selected': smart_text(lookup) == self.lookup_val,
+                'selected': force_text(lookup) == self.lookup_val,
                 'query_string': changelist.get_query_string(
                     {self.lookup_kwarg: lookup}, [self.lookup_kwarg_isnull]
                 ),
@@ -307,6 +309,7 @@ class ChoicesFieldListFilter(FieldListFilter):
                 'display': none_title,
             }
 
+
 FieldListFilter.register(lambda f: bool(f.choices), ChoicesFieldListFilter)
 
 
@@ -376,6 +379,7 @@ class DateFieldListFilter(FieldListFilter):
                 'display': title,
             }
 
+
 FieldListFilter.register(
     lambda f: isinstance(f, models.DateField), DateFieldListFilter)
 
@@ -417,7 +421,7 @@ class AllValuesFieldListFilter(FieldListFilter):
             if val is None:
                 include_none = True
                 continue
-            val = smart_text(val)
+            val = force_text(val)
             yield {
                 'selected': self.lookup_val == val,
                 'query_string': changelist.get_query_string({
@@ -434,6 +438,7 @@ class AllValuesFieldListFilter(FieldListFilter):
                 'display': self.empty_value_display,
             }
 
+
 FieldListFilter.register(lambda f: True, AllValuesFieldListFilter)
 
 
