@@ -1,6 +1,5 @@
 import inspect
 import os
-import re
 from importlib import import_module
 
 from django.apps import apps
@@ -8,11 +7,15 @@ from django.conf import settings
 from django.contrib import admin
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.admindocs import utils
-from django.core import urlresolvers
+from django.contrib.admindocs.utils import (
+    replace_named_groups, replace_unnamed_groups,
+)
 from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
 from django.db import models
 from django.http import Http404
 from django.template.engine import Engine
+from django.urls import get_mod_func, get_resolver, get_urlconf, reverse
+from django.utils import six
 from django.utils.decorators import method_decorator
 from django.utils.inspect import (
     func_accepts_kwargs, func_accepts_var_args, func_has_no_args,
@@ -38,7 +41,7 @@ class BaseAdminDocsView(TemplateView):
         return super(BaseAdminDocsView, self).dispatch(request, *args, **kwargs)
 
     def get_context_data(self, **kwargs):
-        kwargs.update({'root_path': urlresolvers.reverse('admin:index')})
+        kwargs.update({'root_path': reverse('admin:index')})
         kwargs.update(admin.site.each_context(self.request))
         return super(BaseAdminDocsView, self).get_context_data(**kwargs)
 
@@ -126,13 +129,23 @@ class TemplateFilterIndexView(BaseAdminDocsView):
 class ViewIndexView(BaseAdminDocsView):
     template_name = 'admin_doc/view_index.html'
 
+    @staticmethod
+    def _get_full_name(func):
+        mod_name = func.__module__
+        if six.PY3:
+            return '%s.%s' % (mod_name, func.__qualname__)
+        else:
+            # PY2 does not support __qualname__
+            func_name = getattr(func, '__name__', func.__class__.__name__)
+            return '%s.%s' % (mod_name, func_name)
+
     def get_context_data(self, **kwargs):
         views = []
         urlconf = import_module(settings.ROOT_URLCONF)
         view_functions = extract_views_from_urlpatterns(urlconf.urlpatterns)
         for (func, regex, namespace, name) in view_functions:
             views.append({
-                'full_name': '%s.%s' % (func.__module__, getattr(func, '__name__', func.__class__.__name__)),
+                'full_name': self._get_full_name(func),
                 'url': simplify_regex(regex),
                 'url_name': ':'.join((namespace or []) + (name and [name] or [])),
                 'namespace': ':'.join((namespace or [])),
@@ -145,13 +158,34 @@ class ViewIndexView(BaseAdminDocsView):
 class ViewDetailView(BaseAdminDocsView):
     template_name = 'admin_doc/view_detail.html'
 
+    @staticmethod
+    def _get_view_func(view):
+        urlconf = get_urlconf()
+        if get_resolver(urlconf)._is_callback(view):
+            mod, func = get_mod_func(view)
+            try:
+                # Separate the module and function, e.g.
+                # 'mymodule.views.myview' -> 'mymodule.views', 'myview').
+                return getattr(import_module(mod), func)
+            except ImportError:
+                # Import may fail because view contains a class name, e.g.
+                # 'mymodule.views.ViewContainer.my_view', so mod takes the form
+                # 'mymodule.views.ViewContainer'. Parse it again to separate
+                # the module and class.
+                mod, klass = get_mod_func(mod)
+                return getattr(getattr(import_module(mod), klass), func)
+            except AttributeError:
+                # PY2 generates incorrect paths for views that are methods,
+                # e.g. 'mymodule.views.ViewContainer.my_view' will be
+                # listed as 'mymodule.views.my_view' because the class name
+                # can't be detected. This causes an AttributeError when
+                # trying to resolve the view.
+                return None
+
     def get_context_data(self, **kwargs):
         view = self.kwargs['view']
-        urlconf = urlresolvers.get_urlconf()
-        if urlresolvers.get_resolver(urlconf)._is_callback(view):
-            mod, func = urlresolvers.get_mod_func(view)
-            view_func = getattr(import_module(mod), func)
-        else:
+        view_func = self._get_view_func(view)
+        if view_func is None:
             raise Http404
         title, body, metadata = utils.parse_docstring(view_func.__doc__)
         if title:
@@ -268,7 +302,6 @@ class ModelDetailView(BaseAdminDocsView):
                     })
                 else:
                     arguments = get_func_full_args(func)
-                    print_arguments = arguments
                     # Join arguments with ', ' and in case of default value,
                     # join it with '='. Use repr() so that strings will be
                     # correctly displayed.
@@ -324,10 +357,15 @@ class TemplateDetailView(BaseAdminDocsView):
             # This doesn't account for template loaders (#24128).
             for index, directory in enumerate(default_engine.dirs):
                 template_file = os.path.join(directory, template)
+                if os.path.exists(template_file):
+                    with open(template_file) as f:
+                        template_contents = f.read()
+                else:
+                    template_contents = ''
                 templates.append({
                     'file': template_file,
                     'exists': os.path.exists(template_file),
-                    'contents': lambda: open(template_file).read() if os.path.exists(template_file) else '',
+                    'contents': template_contents,
                     'order': index,
                 })
         kwargs.update({
@@ -388,24 +426,17 @@ def extract_views_from_urlpatterns(urlpatterns, base='', namespace=None):
             raise TypeError(_("%s does not appear to be a urlpattern object") % p)
     return views
 
-named_group_matcher = re.compile(r'\(\?P(<\w+>).+?\)')
-non_named_group_matcher = re.compile(r'\(.*?\)')
-
 
 def simplify_regex(pattern):
+    r"""
+    Clean up urlpattern regexes into something more readable by humans. For
+    example, turn "^(?P<sport_slug>\w+)/athletes/(?P<athlete_slug>\w+)/$"
+    into "/<sport_slug>/athletes/<athlete_slug>/".
     """
-    Clean up urlpattern regexes into something somewhat readable by Mere Humans:
-    turns something like "^(?P<sport_slug>\w+)/athletes/(?P<athlete_slug>\w+)/$"
-    into "<sport_slug>/athletes/<athlete_slug>/"
-    """
-    # handle named groups first
-    pattern = named_group_matcher.sub(lambda m: m.group(1), pattern)
-
-    # handle non-named groups
-    pattern = non_named_group_matcher.sub("<var>", pattern)
-
+    pattern = replace_named_groups(pattern)
+    pattern = replace_unnamed_groups(pattern)
     # clean up any outstanding regex-y characters.
-    pattern = pattern.replace('^', '').replace('$', '').replace('?', '').replace('//', '/').replace('\\', '')
+    pattern = pattern.replace('^', '').replace('$', '').replace('?', '')
     if not pattern.startswith('/'):
         pattern = '/' + pattern
     return pattern
