@@ -4,6 +4,7 @@ import datetime
 import decimal
 import hashlib
 import logging
+import re
 from time import time
 
 from django.conf import settings
@@ -36,9 +37,9 @@ class CursorWrapper(object):
         return self
 
     def __exit__(self, type, value, traceback):
-        # Ticket #17671 - Close instead of passing thru to avoid backend
-        # specific behavior. Catch errors liberally because errors in cleanup
-        # code aren't useful.
+        # Close instead of passing through to avoid backend-specific behavior
+        # (#17671). Catch errors liberally because errors in cleanup code
+        # aren't useful.
         try:
             self.close()
         except self.db.Database.Error:
@@ -180,13 +181,19 @@ def rev_typecast_decimal(d):
 
 
 def truncate_name(name, length=None, hash_len=4):
-    """Shortens a string to a repeatable mangled version with the given length.
     """
-    if length is None or len(name) <= length:
+    Shorten a string to a repeatable mangled version with the given length.
+    If a quote stripped name contains a username, e.g. USERNAME"."TABLE,
+    truncate the table portion only.
+    """
+    match = re.match(r'([^"]+)"\."([^"]+)', name)
+    table_name = match.group(2) if match else name
+
+    if length is None or len(table_name) <= length:
         return name
 
-    hsh = hashlib.md5(force_bytes(name)).hexdigest()[:hash_len]
-    return '%s%s' % (name[:length - hash_len], hsh)
+    hsh = hashlib.md5(force_bytes(table_name)).hexdigest()[:hash_len]
+    return '%s%s%s' % (match.group(1) + '"."' if match else '', table_name[:length - hash_len], hsh)
 
 
 def format_number(value, max_digits, decimal_places):
@@ -209,3 +216,13 @@ def format_number(value, max_digits, decimal_places):
     if decimal_places is not None:
         return "%.*f" % (decimal_places, value)
     return "{:f}".format(value)
+
+
+def strip_quotes(table_name):
+    """
+    Strip quotes off of quoted table names to make them safe for use in index
+    names, sequence names, etc. For example '"USER"."TABLE"' (an Oracle naming
+    scheme) becomes 'USER"."TABLE'.
+    """
+    has_quotes = table_name.startswith('"') and table_name.endswith('"')
+    return table_name[1:-1] if has_quotes else table_name
