@@ -1,23 +1,22 @@
 from __future__ import unicode_literals
 
 import datetime
+import warnings
 
 from django.forms.utils import flatatt, pretty_name
 from django.forms.widgets import Textarea, TextInput
 from django.utils import six
-from django.utils.encoding import (
-    force_text, python_2_unicode_compatible, smart_text,
-)
+from django.utils.deprecation import RemovedInDjango21Warning
+from django.utils.encoding import force_text, python_2_unicode_compatible
+from django.utils.functional import cached_property
 from django.utils.html import conditional_escape, format_html, html_safe
+from django.utils.inspect import func_supports_parameter
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 
 __all__ = ('BoundField',)
 
 
-UNSET = object()
-
-
 @html_safe
 @python_2_unicode_compatible
 class BoundField(object):
@@ -34,7 +33,6 @@ class BoundField(object):
         else:
             self.label = self.field.label
         self.help_text = field.help_text or ''
-        self._initial_value = UNSET
 
     def __str__(self):
         """Renders this field as an HTML widget."""
@@ -42,28 +40,42 @@ class BoundField(object):
             return self.as_widget() + self.as_hidden(only_initial=True)
         return self.as_widget()
 
-    def __iter__(self):
+    @cached_property
+    def subwidgets(self):
         """
-        Yields rendered strings that comprise all widgets in this BoundField.
+        Most widgets yield a single subwidget, but others like RadioSelect and
+        CheckboxSelectMultiple produce one subwidget for each choice.
 
-        This really is only useful for RadioSelect widgets, so that you can
-        iterate over individual radio buttons in a template.
+        This property is cached so that only one database query occurs when
+        rendering ModelChoiceFields.
         """
         id_ = self.field.widget.attrs.get('id') or self.auto_id
         attrs = {'id': id_} if id_ else {}
         attrs = self.build_widget_attrs(attrs)
-        for subwidget in self.field.widget.subwidgets(self.html_name, self.value(), attrs):
-            yield subwidget
+        return list(
+            BoundWidget(self.field.widget, widget, self.form.renderer)
+            for widget in self.field.widget.subwidgets(self.html_name, self.value(), attrs=attrs)
+        )
+
+    def __bool__(self):
+        # BoundField evaluates to True even if it doesn't have subwidgets.
+        return True
+
+    def __nonzero__(self):      # Python 2 compatibility
+        return type(self).__bool__(self)
+
+    def __iter__(self):
+        return iter(self.subwidgets)
 
     def __len__(self):
-        return len(list(self.__iter__()))
+        return len(self.subwidgets)
 
     def __getitem__(self, idx):
         # Prevent unnecessary reevaluation when accessing BoundField's attrs
         # from templates.
         if not isinstance(idx, six.integer_types + (slice,)):
             raise TypeError
-        return list(self.__iter__())[idx]
+        return self.subwidgets[idx]
 
     @property
     def errors(self):
@@ -98,7 +110,23 @@ class BoundField(object):
             name = self.html_name
         else:
             name = self.html_initial_name
-        return force_text(widget.render(name, self.value(), attrs=attrs))
+
+        kwargs = {}
+        if func_supports_parameter(widget.render, 'renderer'):
+            kwargs['renderer'] = self.form.renderer
+        else:
+            warnings.warn(
+                'Add the `renderer` argument to the render() method of %s. '
+                'It will be mandatory in Django 2.1.' % widget.__class__,
+                RemovedInDjango21Warning, stacklevel=2,
+            )
+        html = widget.render(
+            name=name,
+            value=self.value(),
+            attrs=attrs,
+            **kwargs
+        )
+        return force_text(html)
 
     def as_text(self, attrs=None, **kwargs):
         """
@@ -128,12 +156,9 @@ class BoundField(object):
         Returns the value for this BoundField, using the initial value if
         the form is not bound or the data otherwise.
         """
-        if not self.form.is_bound:
-            data = self.initial
-        else:
-            data = self.field.bound_data(
-                self.data, self.form.initial.get(self.name, self.field.initial)
-            )
+        data = self.initial
+        if self.form.is_bound:
+            data = self.field.bound_data(self.data, data)
         return self.field.prepare_value(data)
 
     def label_tag(self, contents=None, attrs=None, label_suffix=None):
@@ -198,8 +223,8 @@ class BoundField(object):
         associated Form has specified auto_id. Returns an empty string otherwise.
         """
         auto_id = self.form.auto_id
-        if auto_id and '%s' in smart_text(auto_id):
-            return smart_text(auto_id) % self.html_name
+        if auto_id and '%s' in force_text(auto_id):
+            return force_text(auto_id) % self.html_name
         elif auto_id:
             return self.html_name
         return ''
@@ -215,20 +240,14 @@ class BoundField(object):
         id_ = widget.attrs.get('id') or self.auto_id
         return widget.id_for_label(id_)
 
-    @property
+    @cached_property
     def initial(self):
-        data = self.form.initial.get(self.name, self.field.initial)
-        if callable(data):
-            if self._initial_value is not UNSET:
-                data = self._initial_value
-            else:
-                data = data()
-                # If this is an auto-generated default date, nix the
-                # microseconds for standardized handling. See #22502.
-                if (isinstance(data, (datetime.datetime, datetime.time)) and
-                        not self.field.widget.supports_microseconds):
-                    data = data.replace(microsecond=0)
-                self._initial_value = data
+        data = self.form.get_initial_for_field(self.field, self.name)
+        # If this is an auto-generated default date, nix the microseconds for
+        # standardized handling. See #22502.
+        if (isinstance(data, (datetime.datetime, datetime.time)) and
+                not self.field.widget.supports_microseconds):
+            data = data.replace(microsecond=0)
         return data
 
     def build_widget_attrs(self, attrs, widget=None):
@@ -240,3 +259,45 @@ class BoundField(object):
         if self.field.disabled:
             attrs['disabled'] = True
         return attrs
+
+
+@html_safe
+@python_2_unicode_compatible
+class BoundWidget(object):
+    """
+    A container class used for iterating over widgets. This is useful for
+    widgets that have choices. For example, the following can be used in a
+    template:
+
+    {% for radio in myform.beatles %}
+      <label for="{{ radio.id_for_label }}">
+        {{ radio.choice_label }}
+        <span class="radio">{{ radio.tag }}</span>
+      </label>
+    {% endfor %}
+    """
+    def __init__(self, parent_widget, data, renderer):
+        self.parent_widget = parent_widget
+        self.data = data
+        self.renderer = renderer
+
+    def __str__(self):
+        return self.tag(wrap_label=True)
+
+    def tag(self, wrap_label=False):
+        context = {'widget': self.data, 'wrap_label': wrap_label}
+        return self.parent_widget._render(self.template_name, context, self.renderer)
+
+    @property
+    def template_name(self):
+        if 'template_name' in self.data:
+            return self.data['template_name']
+        return self.parent_widget.template_name
+
+    @property
+    def id_for_label(self):
+        return 'id_%s_%s' % (self.data['name'], self.data['index'])
+
+    @property
+    def choice_label(self):
+        return self.data['label']
