@@ -134,7 +134,7 @@ def floatformat(text, arg=-1):
     """
 
     try:
-        input_val = force_text(text)
+        input_val = repr(text)
         d = Decimal(input_val)
     except UnicodeEncodeError:
         return ''
@@ -165,7 +165,8 @@ def floatformat(text, arg=-1):
     try:
         # Set the precision high enough to avoid an exception, see #15789.
         tupl = d.as_tuple()
-        units = len(tupl[1]) - tupl[2]
+        units = len(tupl[1])
+        units += -tupl[2] if m else tupl[2]
         prec = abs(p) + units + 1
 
         # Avoid conversion to scientific notation by accessing `sign`, `digits`
@@ -259,7 +260,7 @@ def stringformat(value, arg):
 def title(value):
     """Converts a string into titlecase."""
     t = re.sub("([a-z])'([A-Z])", lambda m: m.group(0).lower(), value.title())
-    return re.sub("\d([A-Z])", lambda m: m.group(0).lower(), t)
+    return re.sub(r"\d([A-Z])", lambda m: m.group(0).lower(), t)
 
 
 @register.filter(is_safe=True)
