@@ -1,11 +1,8 @@
 from __future__ import unicode_literals
 
 import difflib
-import errno
 import json
-import os
 import posixpath
-import socket
 import sys
 import threading
 import unittest
@@ -19,7 +16,7 @@ from unittest.util import safe_repr
 from django.apps import apps
 from django.conf import settings
 from django.core import mail
-from django.core.exceptions import ImproperlyConfigured, ValidationError
+from django.core.exceptions import ValidationError
 from django.core.files import locks
 from django.core.handlers.wsgi import WSGIHandler, get_path_info
 from django.core.management import call_command
@@ -29,6 +26,7 @@ from django.core.servers.basehttp import WSGIRequestHandler, WSGIServer
 from django.db import DEFAULT_DB_ALIAS, connection, connections, transaction
 from django.forms.fields import CharField
 from django.http import QueryDict
+from django.http.request import split_domain_port, validate_host
 from django.test.client import Client
 from django.test.html import HTMLParseError, parse_html
 from django.test.signals import setting_changed, template_rendered
@@ -177,6 +175,7 @@ class SimpleTestCase(unittest.TestCase):
             for alias in connections:
                 connection = connections[alias]
                 connection.cursor = _CursorFailure(cls.__name__, connection.cursor)
+                connection.chunked_cursor = _CursorFailure(cls.__name__, connection.chunked_cursor)
 
     @classmethod
     def tearDownClass(cls):
@@ -184,6 +183,7 @@ class SimpleTestCase(unittest.TestCase):
             for alias in connections:
                 connection = connections[alias]
                 connection.cursor = connection.cursor.wrapped
+                connection.chunked_cursor = connection.chunked_cursor.wrapped
         if hasattr(cls, '_cls_modified_context'):
             cls._cls_modified_context.disable()
             delattr(cls, '_cls_modified_context')
@@ -303,6 +303,17 @@ class SimpleTestCase(unittest.TestCase):
                 path = urljoin(response.request['PATH_INFO'], path)
 
             if fetch_redirect_response:
+                # netloc might be empty, or in cases where Django tests the
+                # HTTP scheme, the convention is for netloc to be 'testserver'.
+                # Trust both as "internal" URLs here.
+                domain, port = split_domain_port(netloc)
+                if domain and not validate_host(domain, settings.ALLOWED_HOSTS):
+                    raise ValueError(
+                        "The test client is unable to fetch remote URLs (got %s). "
+                        "If the host is served by Django, add '%s' to ALLOWED_HOSTS. "
+                        "Otherwise, use assertRedirects(..., fetch_redirect_response=False)."
+                        % (url, domain)
+                    )
                 redirect_response = response.client.get(path, QueryDict(query), secure=(scheme == 'https'))
 
                 # Get the redirection page, using the same client that was used
@@ -793,6 +804,12 @@ class SimpleTestCase(unittest.TestCase):
                 standardMsg = '%s == %s' % (safe_repr(xml1, True), safe_repr(xml2, True))
                 self.fail(self._formatMessage(msg, standardMsg))
 
+    if six.PY2:
+        assertCountEqual = unittest.TestCase.assertItemsEqual
+        assertNotRegex = unittest.TestCase.assertNotRegexpMatches
+        assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
+        assertRegex = unittest.TestCase.assertRegexpMatches
+
 
 class TransactionTestCase(SimpleTestCase):
 
@@ -1074,11 +1091,23 @@ class TestCase(TransactionTestCase):
 
 class CheckCondition(object):
     """Descriptor class for deferred condition checking"""
-    def __init__(self, cond_func):
-        self.cond_func = cond_func
+    def __init__(self, *conditions):
+        self.conditions = conditions
+
+    def add_condition(self, condition, reason):
+        return self.__class__(*self.conditions + ((condition, reason),))
 
     def __get__(self, instance, cls=None):
-        return self.cond_func()
+        # Trigger access for all bases.
+        if any(getattr(base, '__unittest_skip__', False) for base in cls.__bases__):
+            return True
+        for condition, reason in self.conditions:
+            if condition():
+                # Override this descriptor's value and set the skip reason.
+                cls.__unittest_skip__ = True
+                cls.__unittest_skip_why__ = reason
+                return True
+        return False
 
 
 def _deferredSkip(condition, reason):
@@ -1094,8 +1123,13 @@ def _deferredSkip(condition, reason):
         else:
             # Assume a class is decorated
             test_item = test_func
-            test_item.__unittest_skip__ = CheckCondition(condition)
-        test_item.__unittest_skip_why__ = reason
+            # Retrieve the possibly existing value from the class's dict to
+            # avoid triggering the descriptor.
+            skip = test_func.__dict__.get('__unittest_skip__')
+            if isinstance(skip, CheckCondition):
+                test_item.__unittest_skip__ = skip.add_condition(condition, reason)
+            elif skip is not True:
+                test_item.__unittest_skip__ = CheckCondition((condition, reason))
         return test_item
     return decorator
 
@@ -1223,10 +1257,9 @@ class LiveServerThread(threading.Thread):
     Thread for running a live http server while the tests are running.
     """
 
-    def __init__(self, host, possible_ports, static_handler, connections_override=None):
+    def __init__(self, host, static_handler, connections_override=None, port=0):
         self.host = host
-        self.port = None
-        self.possible_ports = possible_ports
+        self.port = port
         self.is_ready = threading.Event()
         self.error = None
         self.static_handler = static_handler
@@ -1246,43 +1279,28 @@ class LiveServerThread(threading.Thread):
         try:
             # Create the handler for serving static and media files
             handler = self.static_handler(_MediaFilesHandler(WSGIHandler()))
-
-            # Go through the list of possible ports, hoping that we can find
-            # one that is free to use for the WSGI server.
-            for index, port in enumerate(self.possible_ports):
-                try:
-                    self.httpd = self._create_server(port)
-                except socket.error as e:
-                    if (index + 1 < len(self.possible_ports) and
-                            e.errno == errno.EADDRINUSE):
-                        # This port is already in use, so we go on and try with
-                        # the next one in the list.
-                        continue
-                    else:
-                        # Either none of the given ports are free or the error
-                        # is something else than "Address already in use". So
-                        # we let that error bubble up to the main thread.
-                        raise
-                else:
-                    # A free port was found.
-                    self.port = port
-                    break
-
+            self.httpd = self._create_server()
+            # If binding to port zero, assign the port allocated by the OS.
+            if self.port == 0:
+                self.port = self.httpd.server_address[1]
             self.httpd.set_app(handler)
             self.is_ready.set()
             self.httpd.serve_forever()
         except Exception as e:
             self.error = e
             self.is_ready.set()
+        finally:
+            connections.close_all()
 
-    def _create_server(self, port):
-        return WSGIServer((self.host, port), QuietWSGIRequestHandler, allow_reuse_address=False)
+    def _create_server(self):
+        return WSGIServer((self.host, self.port), QuietWSGIRequestHandler, allow_reuse_address=False)
 
     def terminate(self):
         if hasattr(self, 'httpd'):
             # Stop the WSGI server
             self.httpd.shutdown()
             self.httpd.server_close()
+        self.join()
 
 
 class LiveServerTestCase(TransactionTestCase):
@@ -1296,13 +1314,14 @@ class LiveServerTestCase(TransactionTestCase):
     sqlite) and each thread needs to commit all their transactions so that the
     other thread can see the changes.
     """
-
+    host = 'localhost'
+    port = 0
+    server_thread_class = LiveServerThread
     static_handler = _StaticFilesHandler
 
     @classproperty
     def live_server_url(cls):
-        return 'http://%s:%s' % (
-            cls.server_thread.host, cls.server_thread.port)
+        return 'http://%s:%s' % (cls.host, cls.server_thread.port)
 
     @classmethod
     def setUpClass(cls):
@@ -1311,36 +1330,16 @@ class LiveServerTestCase(TransactionTestCase):
         for conn in connections.all():
             # If using in-memory sqlite databases, pass the connections to
             # the server thread.
-            if conn.vendor == 'sqlite' and conn.is_in_memory_db(conn.settings_dict['NAME']):
+            if conn.vendor == 'sqlite' and conn.is_in_memory_db():
                 # Explicitly enable thread-shareability for this connection
                 conn.allow_thread_sharing = True
                 connections_override[conn.alias] = conn
 
-        # Launch the live server's thread
-        specified_address = os.environ.get(
-            'DJANGO_LIVE_TEST_SERVER_ADDRESS', 'localhost:8081-8179')
-
-        # The specified ports may be of the form '8000-8010,8080,9200-9300'
-        # i.e. a comma-separated list of ports or ranges of ports, so we break
-        # it down into a detailed list of all possible ports.
-        possible_ports = []
-        try:
-            host, port_ranges = specified_address.split(':')
-            for port_range in port_ranges.split(','):
-                # A port range can be of either form: '8000' or '8000-8010'.
-                extremes = list(map(int, port_range.split('-')))
-                assert len(extremes) in [1, 2]
-                if len(extremes) == 1:
-                    # Port range of the form '8000'
-                    possible_ports.append(extremes[0])
-                else:
-                    # Port range of the form '8000-8010'
-                    for port in range(extremes[0], extremes[1] + 1):
-                        possible_ports.append(port)
-        except Exception:
-            msg = 'Invalid address ("%s") for live server.' % specified_address
-            six.reraise(ImproperlyConfigured, ImproperlyConfigured(msg), sys.exc_info()[2])
-        cls.server_thread = cls._create_server_thread(host, possible_ports, connections_override)
+        cls._live_server_modified_settings = modify_settings(
+            ALLOWED_HOSTS={'append': cls.host},
+        )
+        cls._live_server_modified_settings.enable()
+        cls.server_thread = cls._create_server_thread(connections_override)
         cls.server_thread.daemon = True
         cls.server_thread.start()
 
@@ -1353,12 +1352,12 @@ class LiveServerTestCase(TransactionTestCase):
             raise cls.server_thread.error
 
     @classmethod
-    def _create_server_thread(cls, host, possible_ports, connections_override):
-        return LiveServerThread(
-            host,
-            possible_ports,
+    def _create_server_thread(cls, connections_override):
+        return cls.server_thread_class(
+            cls.host,
             cls.static_handler,
             connections_override=connections_override,
+            port=cls.port,
         )
 
     @classmethod
@@ -1368,16 +1367,16 @@ class LiveServerTestCase(TransactionTestCase):
         if hasattr(cls, 'server_thread'):
             # Terminate the live server's thread
             cls.server_thread.terminate()
-            cls.server_thread.join()
 
         # Restore sqlite in-memory database connections' non-shareability
         for conn in connections.all():
-            if conn.vendor == 'sqlite' and conn.is_in_memory_db(conn.settings_dict['NAME']):
+            if conn.vendor == 'sqlite' and conn.is_in_memory_db():
                 conn.allow_thread_sharing = False
 
     @classmethod
     def tearDownClass(cls):
         cls._tearDownClassInternal()
+        cls._live_server_modified_settings.disable()
         super(LiveServerTestCase, cls).tearDownClass()
 
 
